import _mssql

import os


def error_log(str,file_name="error.txt"):
    if not os.path.exists("error.txt"):
        # send_message("etl_sql")
        # print("发送短信通知！")
        pass

    fo = open("error.txt", "w")
    fo.write( str)
    
    # 关闭打开的文件
    fo.close()

def mssql_execute(db_config,SQLStr):
    conn = _mssql.connect(server=db_config.server, user=db_config.user, password=db_config.password, database=db_config.database,charset='utf8')
    try:
        conn.execute_non_query(SQLStr)
    except:
        error_log(SQLStr)
    finally:
        conn.close()




import binascii
from c_sql import Sql_Query



class Sql_Mssql(Sql_Query):
    # Python二进制转SQLbinary
    def binary_to_str(self,binary):
        if binary is not None and binary != 0:
            binarystr = str('0x'.encode('ascii') + binascii.hexlify(binary))
            binarystr = binarystr[2:len(binarystr)-1]
        else:
            binarystr = "0"
        return binarystr
    

    def pymssql_type_to_db_type(self,pymssql_type):
        dict_db_type={1:"string",2:"binary",3:"float",4:"datetime",9:"expression"}
        if pymssql_type in dict_db_type:
            db_type=dict_db_type[pymssql_type]
        else:
            db_type="string"
        return db_type

    def get_colums_value_str(self, column_value, column_type):
        if str(column_value).upper() == "NULL" or column_value is None:
            column_value = f"Null"
        elif column_type in ("string", "text", "datetime", "date", "time"):
            if column_type in ["string", "text"] and "'" in str(column_value):
                column_value = str(str(column_value).rstrip()
                                   ).replace("'", "''")
            elif column_type == "datetime" and len(str(column_value)) > 23:
                column_value = f"""{str(column_value)[0:23]}"""
            column_value = f"""'{column_value}'"""
        elif column_type in ("binary", "bolean", "float", "int", "expression"):
            if column_type == "binary":
                column_value = f"{self.binary_to_str(column_value)}"
            if column_type == "bolean":
                if column_value == False or str(column_value) == 'FALSE':
                    column_value = 0
                elif column_type == True or str(column_value) == "TRUE":
                    column_value = 1
        return column_value





import _mssql
from c_mssql.mssql_sql import Sql_Mssql

class Mssql_Source(Sql_Mssql):
    def __init__(self,db_config):
        self.db_config=db_config

    def get_conn_title(self,data_conn):
        title_dict={}
        for rows in data_conn.get_header():
            title_dict[rows[0]]=self.pymssql_type_to_db_type(rows[1])
        return title_dict

    def get_sql_title(self,sql_str):
        conn = _mssql.connect(server=self.db_config.server, user=self.db_config.user, password=self.db_config.password, database=self.db_config.database,charset='utf8')
        try:
            conn.execute_query(sql_str)
        except:
            raise(f"""sql运行出错{sql_str}""")

        title_dict=self.get_conn_title(conn)
        conn.close()
        return title_dict

    def get_conn(self,sql_str,with_title=False):
        conn = _mssql.connect(server=self.db_config.server, user=self.db_config.user, password=self.db_config.password, database=self.db_config.database,charset='utf8')
        try:
            conn.execute_query(sql_str)
        except:
            raise(f"""sql运行出错{sql_str}""")
        if with_title:
            title_dict=self.get_conn_title(conn)
            return title_dict,conn
        else:
            return conn

    def get_datalist(self,sql_str,with_title=False):
        conn = _mssql.connect(server=self.db_config.server, user=self.db_config.user, password=self.db_config.password, database=self.db_config.database,charset='utf8')
        try:
            conn.execute_query(sql_str)
        except:
            raise(f"""sql运行出错{sql_str}""")
        
        title_dict=self.get_conn_title(conn)
        data_list=[]
        for row in conn:
            row_dict={}
            for title in title_dict:
                row_dict[title]=row[title]
            data_list.append(row_dict)
        conn.close()
        if with_title:
            return title_dict,data_list
        else:
            return data_list


    def get_rowdict(self,sql_str):
        conn = _mssql.connect(server=self.db_config.server, user=self.db_config.user, password=self.db_config.password, database=self.db_config.database,charset='utf8')
        row_dict=conn.execute_row(sql_str)
        conn.close()
        return row_dict

    def get_value(self,sql_str):
        conn = _mssql.connect(server=self.db_config.server, user=self.db_config.user, password=self.db_config.password, database=self.db_config.database,charset='utf8')
        scalar=conn.execute_scalar(sql_str)
        conn.close()
        return scalar



import time, threading
from c_mssql.mssql_source import Mssql_Source
from c_mssql.mssql_execute import mssql_execute

lock = threading.Lock()
taskcount=0

class Mssql_Dst(Mssql_Source):
    """description of class"""
    def __init__(self,db_config,run_task=2):
        self.db_config=db_config
        self.run_task=run_task
        
    #行数据处理
    def row_process(self,dict_row):
        return dict_row

    #带锁的数据插入
    def run_import(self,dst_db,titleStr,sql_datastr_list):
        global taskcount
        taskcount+=1
        lock.acquire()
        try:
            #标题和数据行
            sql_str_data="Union All\n".join(sql_datastr_list)
            mssql_execute(dst_db,titleStr+"\n"+ sql_str_data)
        finally:
            lock.release()
            taskcount-=1
    
    def import_into_dst(self,data_conn,dst_table,dst_title=None,source_title=None,batch=1000):
 
        #如果目标表字段未给定
        if dst_title is None:
            sql_str=f"""select * from {dst_table} with(nolock) where 1=2"""
            dst_title=self.get_sql_title(sql_str)

        elif type(dst_title)==list:
            sql_str=f"""select {",".join(dst_title)} from {dst_table} with(nolock) where 1=2"""
            dst_title_temp=self.get_sql_title(sql_str)
            dst_title_dict={}
            for title in dst_title:
                dst_title_dict[title]=dst_title_temp[title]
            dst_title=dst_title_dict

        if source_title is None:
            source_title=dst_title

        elif type(source_title)==list:
            source_title_dict={}
            for title in source_title:
                if title in dst_title:
                    source_title_dict[title]=dst_title[title]
                else:
                    source_title_dict[title]="string"
            source_title=source_title_dict


        
        #插入数据的Insert头部语句
        title_str=self.get_insert_title_str(dst_table,dst_title)
        #待插入的数据
        sql_str_data_list=[]
        #行计数 
        rowcount=0
        global taskcount
        for row in data_conn:
            rowcount+=1
            dict_source_row={}
            #获取数据单行
            for title in source_title:
                dict_source_row[title]=row[title]
            #对每行数据进行处理
            dst_row_dict=self.row_process(dict_source_row)
            sql_str_data=self.get_select_row_str(dst_title, dst_row_dict)+"\n"
            sql_str_data_list.append(sql_str_data)
            if len(sql_str_data_list)==batch:
                while taskcount>self.run_task:
                    time.sleep(0.01)
                t1 = threading.Thread(target=self.run_import, args=(self.db_config,title_str,sql_str_data_list,))
                t1.start()
                sql_str_data_list=[]


        if len(sql_str_data_list)>0:
            t1 = threading.Thread(target=self.run_import, args=(self.db_config,title_str,sql_str_data_list,))
            t1.start()
        #等待任务结束
        while taskcount>0:
            time.sleep(0.01)
            pass
        return rowcount






from setuptools import setup, find_packages  

# with open("tencent_message\README.md", "r",encoding="utf-8") as fh:
#     long_description = fh.read()
long_description="sql query package"
setup(  
    name = 'c_mssql',  
    version = '0.0.8',
    # keywords = ('chinesename',),  
    description = 'c_mssql',  
    long_description=long_description,
    long_description_content_type="text/markdown",
    license = 'MIT tencent_message',  
    install_requires = ["pymssql"],  
    packages = ['c_mssql'],  # 要打包的项目文件夹
    include_package_data=True,   # 自动打包文件夹内所有数据
    author = 'evanyang',  
    author_email = 'lightyiyi@qq.com',
    url = 'https://www.cnblogs.com/Evan-fanfan/',
    # packages = find_packages(include=("*"),),  
)  


# python setup.py check #检查写的有没有问题，有问题就直接报错了
# python setup.py sdist upload #压缩、并打包上传到pip源上，会在当前目录下产生一个dist文件夹
# #里面就是打好的压缩包
 
# python setup.py sdist #只压缩不上传，就是打成tar.gz这种安装包，给别人用的话直接给他安装包就可以了